@@ -472,6 +472,7 @@ void PackageInstallerImpl::FindUpdates()
 
     // clean the user-installation directory
     if (!session->IsAdminMode()
+      && session->IsSharedSetup()
       && session->GetSpecialPath(SpecialPath::UserInstallRoot) != session->GetSpecialPath(SpecialPath::CommonInstallRoot)
       && package.IsInstalled(ConfigurationScope::User)
       && package.IsInstalled(ConfigurationScope::Common))
@@ -1582,7 +1583,7 @@ void PackageInstallerImpl::CheckDependencies(set<string>& packages, const string
   }
 }
 
-// FIXME: duplicate ocode
+// FIXME: duplicate code
 MPMSTATICFUNC(bool) EndsWith(const string& s, const string& suffix)
 {
   return s.length() >= suffix.length() &&
